pub struct AssetPath<'a> { /* private fields */ }Expand description
Represents a path to an asset in a “virtual filesystem”.
Asset paths consist of three main parts:
AssetPath::source: The name of theAssetSourceto load the asset from. This is optional. If one is not set the default source will be used (which is theassetsfolder by default).AssetPath::path: The “virtual filesystem path” pointing to an asset source file.AssetPath::label: An optional “named sub asset”. When assets are loaded, they are allowed to load “sub assets” of any type, which are identified by a named “label”.
Asset paths are generally constructed (and visualized) as strings:
// This loads the `my_scene.scn` base asset from the default asset source.
let scene: Handle<Scene> = asset_server.load("my_scene.scn");
// This loads the `PlayerMesh` labeled asset from the `my_scene.scn` base asset in the default asset source.
let mesh: Handle<Mesh> = asset_server.load("my_scene.scn#PlayerMesh");
// This loads the `my_scene.scn` base asset from a custom 'remote' asset source.
let scene: Handle<Scene> = asset_server.load("remote://my_scene.scn");AssetPath implements From for &'static str, &'static Path, and &'a String,
which allows us to optimize the static cases.
This means that the common case of asset_server.load("my_scene.scn") when it creates and
clones internal owned AssetPaths.
This also means that you should use AssetPath::parse in cases where &str is the explicit type.
Implementations
Sourceimpl<'a> AssetPath<'a>
impl<'a> AssetPath<'a>
Sourcepub fn parse(asset_path: &'a str) -> AssetPath<'a>
pub fn parse(asset_path: &'a str) -> AssetPath<'a>
Creates a new AssetPath from a string in the asset path format:
- An asset at the root:
"scene.gltf" - An asset nested in some folders:
"some/path/scene.gltf" - An asset with a “label”:
"some/path/scene.gltf#Mesh0" - An asset with a custom “source”:
"custom://some/path/scene.gltf#Mesh0"
Prefer From<'static str> for static strings, as this will prevent allocations
and reference counting for AssetPath::into_owned.
Panics
Panics if the asset path is in an invalid format. Use AssetPath::try_parse for a fallible variant
Sourcepub fn try_parse(
asset_path: &'a str,
) -> Result<AssetPath<'a>, ParseAssetPathError>
pub fn try_parse( asset_path: &'a str, ) -> Result<AssetPath<'a>, ParseAssetPathError>
Creates a new AssetPath from a string in the asset path format:
- An asset at the root:
"scene.gltf" - An asset nested in some folders:
"some/path/scene.gltf" - An asset with a “label”:
"some/path/scene.gltf#Mesh0" - An asset with a custom “source”:
"custom://some/path/scene.gltf#Mesh0"
Prefer From<'static str> for static strings, as this will prevent allocations
and reference counting for AssetPath::into_owned.
This will return a ParseAssetPathError if asset_path is in an invalid format.
Sourcepub fn from_path_buf(path_buf: PathBuf) -> AssetPath<'a>
pub fn from_path_buf(path_buf: PathBuf) -> AssetPath<'a>
Sourcepub fn source(&self) -> &AssetSourceId<'_>
pub fn source(&self) -> &AssetSourceId<'_>
Gets the “asset source”, if one was defined. If none was defined, the default source will be used.
Sourcepub fn without_label(&self) -> AssetPath<'_>
pub fn without_label(&self) -> AssetPath<'_>
Gets the path to the asset in the “virtual filesystem” without a label (if a label is currently set).
Sourcepub fn remove_label(&mut self)
pub fn remove_label(&mut self)
Removes a “sub-asset label” from this AssetPath, if one was set.
Sourcepub fn take_label(&mut self) -> Option<CowArc<'a, str>>
pub fn take_label(&mut self) -> Option<CowArc<'a, str>>
Takes the “sub-asset label” from this AssetPath, if one was set.
Sourcepub fn with_label(self, label: impl Into<CowArc<'a, str>>) -> AssetPath<'a>
pub fn with_label(self, label: impl Into<CowArc<'a, str>>) -> AssetPath<'a>
Returns this asset path with the given label. This will replace the previous label if it exists.
Sourcepub fn with_source(self, source: impl Into<AssetSourceId<'a>>) -> AssetPath<'a>
pub fn with_source(self, source: impl Into<AssetSourceId<'a>>) -> AssetPath<'a>
Returns this asset path with the given asset source. This will replace the previous asset source if it exists.
Sourcepub fn parent(&self) -> Option<AssetPath<'a>>
pub fn parent(&self) -> Option<AssetPath<'a>>
Returns an AssetPath for the parent folder of this path, if there is a parent folder in the path.
Sourcepub fn into_owned(self) -> AssetPath<'static>
pub fn into_owned(self) -> AssetPath<'static>
Sourcepub fn clone_owned(&self) -> AssetPath<'static>
pub fn clone_owned(&self) -> AssetPath<'static>
Sourcepub fn resolve(&self, path: &AssetPath<'_>) -> AssetPath<'static>
pub fn resolve(&self, path: &AssetPath<'_>) -> AssetPath<'static>
Resolves an AssetPath relative to self.
Semantics:
- If
pathis label-only (default source, empty path, label set), replaceself’s label. - If
pathbegins with/, treat it as rooted at the asset-source root (not the filesystem). - If
pathhas an explicit source (name://...), it replaces the base source. - Relative segments are concatenated and normalized (
./..removal), preserving extra..if the base underflows.
let base = AssetPath::parse("a/b");
assert_eq!(base.resolve(&AssetPath::parse("c")), AssetPath::parse("a/b/c"));
assert_eq!(base.resolve(&AssetPath::parse("./c")), AssetPath::parse("a/b/c"));
assert_eq!(base.resolve(&AssetPath::parse("../c")), AssetPath::parse("a/c"));
assert_eq!(base.resolve(&AssetPath::parse("c.png")), AssetPath::parse("a/b/c.png"));
assert_eq!(base.resolve(&AssetPath::parse("/c")), AssetPath::parse("c"));
assert_eq!(AssetPath::parse("a/b.png").resolve(&AssetPath::parse("#c")), AssetPath::parse("a/b.png#c"));
assert_eq!(AssetPath::parse("a/b.png#c").resolve(&AssetPath::parse("#d")), AssetPath::parse("a/b.png#d"));See also AssetPath::resolve_str.
Sourcepub fn resolve_embed(&self, path: &AssetPath<'_>) -> AssetPath<'static>
pub fn resolve_embed(&self, path: &AssetPath<'_>) -> AssetPath<'static>
Resolves an AssetPath relative to self using embedded (RFC 1808) semantics.
Semantics:
- Remove the “file portion” of the base before concatenation (unless the base ends with
/). - Otherwise identical to
AssetPath::resolve.
let base = AssetPath::parse("a/b");
assert_eq!(base.resolve_embed(&AssetPath::parse("c")), AssetPath::parse("a/c"));
assert_eq!(base.resolve_embed(&AssetPath::parse("./c")), AssetPath::parse("a/c"));
assert_eq!(base.resolve_embed(&AssetPath::parse("../c")), AssetPath::parse("c"));
assert_eq!(base.resolve_embed(&AssetPath::parse("c.png")), AssetPath::parse("a/c.png"));
assert_eq!(base.resolve_embed(&AssetPath::parse("/c")), AssetPath::parse("c"));
assert_eq!(AssetPath::parse("a/b.png").resolve_embed(&AssetPath::parse("#c")), AssetPath::parse("a/b.png#c"));
assert_eq!(AssetPath::parse("a/b.png#c").resolve_embed(&AssetPath::parse("#d")), AssetPath::parse("a/b.png#d"));See also AssetPath::resolve_embed_str.
Sourcepub fn resolve_str(
&self,
path: &str,
) -> Result<AssetPath<'static>, ParseAssetPathError>
pub fn resolve_str( &self, path: &str, ) -> Result<AssetPath<'static>, ParseAssetPathError>
Parses path as an AssetPath, then resolves it relative to self.
Returns an error if parsing fails.
For more details, see AssetPath::resolve.
Sourcepub fn resolve_embed_str(
&self,
path: &str,
) -> Result<AssetPath<'static>, ParseAssetPathError>
pub fn resolve_embed_str( &self, path: &str, ) -> Result<AssetPath<'static>, ParseAssetPathError>
Parses path as an AssetPath, then resolves it relative to self using embedded
(RFC 1808) semantics.
Returns an error if parsing fails.
For more details, see AssetPath::resolve_embed.
Sourcepub fn get_full_extension(&self) -> Option<&str>
pub fn get_full_extension(&self) -> Option<&str>
Returns the full extension (including multiple ‘.’ values).
Ex: Returns "config.ron" for "my_asset.config.ron"
Also strips out anything following a ? to handle query parameters in URIs
Sourcepub fn get_extension(&self) -> Option<&str>
pub fn get_extension(&self) -> Option<&str>
Returns the extension, excluding multiple . values.
Ex: Returns "ron" for "my_asset.config.ron"
Also strips out anything follow a ? to handle query parameters in URIs.
Sourcepub fn is_unapproved(&self) -> bool
pub fn is_unapproved(&self) -> bool
Returns true if this AssetPath points to a file that is
outside of its AssetSource folder.
Example
// Inside the default AssetSource.
let path = AssetPath::parse("thingy.png");
assert!( ! path.is_unapproved());
let path = AssetPath::parse("gui/thingy.png");
assert!( ! path.is_unapproved());
// Inside a different AssetSource.
let path = AssetPath::parse("embedded://thingy.png");
assert!( ! path.is_unapproved());
// Exits the `AssetSource`s directory.
let path = AssetPath::parse("../thingy.png");
assert!(path.is_unapproved());
let path = AssetPath::parse("folder/../../thingy.png");
assert!(path.is_unapproved());
// This references the linux root directory.
let path = AssetPath::parse("/home/thingy.png");
assert!(path.is_unapproved());Trait Implementations
Sourceimpl<'de> Deserialize<'de> for AssetPath<'static>
impl<'de> Deserialize<'de> for AssetPath<'static>
Sourcefn deserialize<D>(deserializer: D) -> Result<Self, D::Error>where
D: Deserializer<'de>,
fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>where
D: Deserializer<'de>,
Sourceimpl<'a> FromReflect for AssetPath<'a>where
AssetPath<'a>: 'static,
impl<'a> FromReflect for AssetPath<'a>where
AssetPath<'a>: 'static,
Sourcefn from_reflect(reflect: &dyn PartialReflect) -> Option<Self>
fn from_reflect(reflect: &dyn PartialReflect) -> Option<Self>
Self from a reflected value.Sourcefn take_from_reflect(
reflect: Box<dyn PartialReflect>,
) -> Result<Self, Box<dyn PartialReflect>>
fn take_from_reflect( reflect: Box<dyn PartialReflect>, ) -> Result<Self, Box<dyn PartialReflect>>
Self using,
constructing the value using from_reflect if that fails. Read moreSourceimpl<'a> GetTypeRegistration for AssetPath<'a>where
AssetPath<'a>: 'static,
impl<'a> GetTypeRegistration for AssetPath<'a>where
AssetPath<'a>: 'static,
Sourcefn get_type_registration() -> TypeRegistration
fn get_type_registration() -> TypeRegistration
TypeRegistration for this type.Sourcefn register_type_dependencies(_registry: &mut TypeRegistry)
fn register_type_dependencies(_registry: &mut TypeRegistry)
Sourceimpl<'a> PartialReflect for AssetPath<'a>where
AssetPath<'a>: 'static,
impl<'a> PartialReflect for AssetPath<'a>where
AssetPath<'a>: 'static,
Sourcefn get_represented_type_info(&self) -> Option<&'static TypeInfo>
fn get_represented_type_info(&self) -> Option<&'static TypeInfo>
Sourcefn to_dynamic(&self) -> Box<dyn PartialReflect>
fn to_dynamic(&self) -> Box<dyn PartialReflect>
Sourcefn try_apply(&mut self, value: &dyn PartialReflect) -> Result<(), ApplyError>
fn try_apply(&mut self, value: &dyn PartialReflect) -> Result<(), ApplyError>
Sourcefn reflect_kind(&self) -> ReflectKind
fn reflect_kind(&self) -> ReflectKind
Sourcefn reflect_ref(&self) -> ReflectRef<'_>
fn reflect_ref(&self) -> ReflectRef<'_>
Sourcefn reflect_mut(&mut self) -> ReflectMut<'_>
fn reflect_mut(&mut self) -> ReflectMut<'_>
Sourcefn reflect_owned(self: Box<Self>) -> ReflectOwned
fn reflect_owned(self: Box<Self>) -> ReflectOwned
Sourcefn try_into_reflect(
self: Box<Self>,
) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
fn try_into_reflect( self: Box<Self>, ) -> Result<Box<dyn Reflect>, Box<dyn PartialReflect>>
Sourcefn try_as_reflect(&self) -> Option<&dyn Reflect>
fn try_as_reflect(&self) -> Option<&dyn Reflect>
Sourcefn try_as_reflect_mut(&mut self) -> Option<&mut dyn Reflect>
fn try_as_reflect_mut(&mut self) -> Option<&mut dyn Reflect>
Sourcefn into_partial_reflect(self: Box<Self>) -> Box<dyn PartialReflect>
fn into_partial_reflect(self: Box<Self>) -> Box<dyn PartialReflect>
Sourcefn as_partial_reflect(&self) -> &dyn PartialReflect
fn as_partial_reflect(&self) -> &dyn PartialReflect
Sourcefn as_partial_reflect_mut(&mut self) -> &mut dyn PartialReflect
fn as_partial_reflect_mut(&mut self) -> &mut dyn PartialReflect
Sourcefn reflect_hash(&self) -> Option<u64>
fn reflect_hash(&self) -> Option<u64>
Sourcefn reflect_partial_eq(&self, value: &dyn PartialReflect) -> Option<bool>
fn reflect_partial_eq(&self, value: &dyn PartialReflect) -> Option<bool>
Sourcefn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
fn reflect_clone(&self) -> Result<Box<dyn Reflect>, ReflectCloneError>
Self using reflection. Read moreSourcefn apply(&mut self, value: &(dyn PartialReflect + 'static))
fn apply(&mut self, value: &(dyn PartialReflect + 'static))
Sourcefn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
fn reflect_clone_and_take<T>(&self) -> Result<T, ReflectCloneError>
PartialReflect, combines reflect_clone and
take in a useful fashion, automatically constructing an appropriate
ReflectCloneError if the downcast fails.Sourcefn reflect_partial_cmp(
&self,
_value: &(dyn PartialReflect + 'static),
) -> Option<Ordering>
fn reflect_partial_cmp( &self, _value: &(dyn PartialReflect + 'static), ) -> Option<Ordering>
Sourcefn is_dynamic(&self) -> bool
fn is_dynamic(&self) -> bool
Sourceimpl<'a> Reflect for AssetPath<'a>where
AssetPath<'a>: 'static,
impl<'a> Reflect for AssetPath<'a>where
AssetPath<'a>: 'static,
Sourcefn as_any_mut(&mut self) -> &mut dyn Any
fn as_any_mut(&mut self) -> &mut dyn Any
&mut dyn Any. Read moreSourcefn into_reflect(self: Box<Self>) -> Box<dyn Reflect>
fn into_reflect(self: Box<Self>) -> Box<dyn Reflect>
Sourcefn as_reflect(&self) -> &dyn Reflect
fn as_reflect(&self) -> &dyn Reflect
Sourcefn as_reflect_mut(&mut self) -> &mut dyn Reflect
fn as_reflect_mut(&mut self) -> &mut dyn Reflect
Sourceimpl<'a> TypePath for AssetPath<'a>where
AssetPath<'a>: 'static,
impl<'a> TypePath for AssetPath<'a>where
AssetPath<'a>: 'static,
Sourcefn type_path() -> &'static str
fn type_path() -> &'static str
Sourcefn short_type_path() -> &'static str
fn short_type_path() -> &'static str
Sourcefn type_ident() -> Option<&'static str>
fn type_ident() -> Option<&'static str>
Sourcefn crate_name() -> Option<&'static str>
fn crate_name() -> Option<&'static str>
impl<'a> Eq for AssetPath<'a>
impl<'a> StructuralPartialEq for AssetPath<'a>